package sun.java2d.cmm.lcms;

import java.security.AccessController;
import java.security.PrivilegedAction;
import sun.java2d.cmm.ColorTransform;
import java.awt.color.ICC_Profile;
import java.awt.color.CMMException;
import sun.java2d.cmm.Profile;
import sun.java2d.cmm.PCMM;

public class LCMS implements PCMM
{
    private static LCMS theLcms;
    
    @Override
    public Profile loadProfile(byte[] array) {
        Object o = new Object();
        long loadProfileNative = this.loadProfileNative(array, o);
        if (loadProfileNative != 0L) {
            return new LCMSProfile(loadProfileNative, o);
        }
        return null;
    }
    
    private native long loadProfileNative(byte[] p0, Object p1);
    
    private LCMSProfile getLcmsProfile(Profile obj) {
        if ((obj instanceof LCMSProfile)) {
            return (LCMSProfile)obj;
        }
        throw new CMMException("Invalid profile: " + obj);
    }
    
    public void freeProfile(Profile profile) {
    }
    
    public int getProfileSize(Profile profile) {
        synchronized (profile) {
            return this.getProfileSizeNative(this.getLcmsProfile(profile).getLcmsPtr());
        }
    }
    
    private native int getProfileSizeNative(long p0);
    
    public void getProfileData(Profile profile, byte[] array) {
        synchronized (profile) {
            this.getProfileDataNative(this.getLcmsProfile(profile).getLcmsPtr(), array);
        }
    }
    
    private native void getProfileDataNative(long p0, byte[] p1);
    
    public int getTagSize(Profile profile, int n) {
        LCMSProfile lcmsProfile = this.getLcmsProfile(profile);
        synchronized (lcmsProfile) {
            LCMSProfile.TagData tag = lcmsProfile.getTag(n);
            return (tag == null) ? 0 : tag.getSize();
        }
    }
    
    static native byte[] getTagNative(long p0, int p1);
    
    public void getTagData(Profile profile, int n, byte[] array) {
        LCMSProfile lcmsProfile = this.getLcmsProfile(profile);
        synchronized (lcmsProfile) {
            LCMSProfile.TagData tag = lcmsProfile.getTag(n);
            if (tag != null) {
                tag.copyDataTo(array);
            }
        }
    }
    
    @Override
    public synchronized void setTagData(Profile profile, int n, byte[] array) {
        LCMSProfile lcmsProfile = this.getLcmsProfile(profile);
        synchronized (lcmsProfile) {
            lcmsProfile.clearTagCache();
            this.setTagDataNative(lcmsProfile.getLcmsPtr(), n, array);
        }
    }
    
    private native void setTagDataNative(long p0, int p1, byte[] p2);
    
    public static synchronized native LCMSProfile getProfileID(ICC_Profile p0);
    
    static long createTransform(LCMSProfile[] array, int n, int n2, boolean b, int n3, boolean b2, Object o) {
        long[] array2 = new long[array.length];
        int i = 0;
        while (i < array.length) {
            if (array[i] == null) {
                throw new CMMException("Unknown profile ID");
            }
            array2[i] = array[i].getLcmsPtr();
            ++i;
        }
        return createNativeTransform(array2, n, n2, b, n3, b2, o);
    }
    
    private static native long createNativeTransform(long[] p0, int p1, int p2, boolean p3, int p4, boolean p5, Object p6);
    
    public ColorTransform createTransform(ICC_Profile icc_Profile, int n, int n2) {
        return new LCMSTransform(icc_Profile, n, n);
    }
    
    public synchronized ColorTransform createTransform(ColorTransform[] array) {
        return new LCMSTransform(array);
    }
    
    public static native void colorConvert(LCMSTransform p0, LCMSImageLayout p1, LCMSImageLayout p2);
    
    public static native void freeTransform(long p0);
    
    public static native void initLCMS(Class p0, Class p1, Class p2);
    
    private LCMS() {
    }
    
    static synchronized PCMM getModule() {
        if (LCMS.theLcms != null) {
            return LCMS.theLcms;
        }
        AccessController.doPrivileged(new PrivilegedAction() {
            @Override
            public Object run() {
                System.loadLibrary("awt");
                System.loadLibrary("lcms");
                return null;
            }
        });
        initLCMS(LCMSTransform.class, LCMSImageLayout.class, ICC_Profile.class);
        return LCMS.theLcms = new LCMS();
    }
    
    static {
        LCMS.theLcms = null;
    }
}
